package com.caimei365.tools.controller;

import com.caimei365.tools.model.ResponseJson;
import com.caimei365.tools.model.dto.message.*;
import com.caimei365.tools.model.dto.message.enums.MessageType;
import com.caimei365.tools.service.MessageSender;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;

@Api(tags = "消息工具")
@RestController
@RequiredArgsConstructor
@RequestMapping("/tools")
public class MessageApi {

    private final MessageSender messageSender;

    /**
     * 发送消息
     *
     * @param messageModel {
     *                     信息类型枚举类  code;
     *                     mq消息参数     mqInfo;
     *                     消息体        info;
     *                     }
     * @return
     */
    @ApiOperation("综合消息工具")
    @PostMapping("/message/send")
    public ResponseJson sendCommonMessage(@RequestBody MessageModel messageModel) {
        messageSender.messageSend(messageModel);
        return ResponseJson.success();
    }

    @ApiOperation("综合消息工具测试")
    @GetMapping("/message/send/try")
    public ResponseJson sendCommonMessagetry(Integer status,String openId) {
        //    WECHAT_TEMPLATE_MESSAGE(1, "微信模板消息"),
        //    WEB_INSIDE_MESSAGE(2, "站内消息"),
        //    EMAILS(3, "邮件"),
        //    SYS_MESSAGE(4, "短信");
        MessageModel messageModel = new MessageModel();
        if (4 == status) {
            MqInfo messageLine = new MqInfo().topic("MessageLine").async(1).delay(4);
            SmsMessage smsMessage = new SmsMessage().type(3).mobile("15827317748").smsMessageContent("亲，好久没登陆采美了，" +
                    "近期在找新项目吗？采美新品橱窗有多款院线产品和仪器，现在登陆采美https://www.caimei365.com/，可获一张包邮券！退订回T");
            messageModel = messageModel.code(MessageType.SYS_MESSAGE).mqInfo(messageLine).info(smsMessage);
        }
        if (3 == status) {
            EmailMessage subject = new EmailMessage().emailMessageContent("测试接口").cc("z525282777@163.com").receiverAddress("525282777@qq.com").subject("接口测试");
            messageModel = new MessageModel().code(MessageType.EMAILS).info(subject);
        }
        if (2 == status) {
            InsideMessage sud = new InsideMessage().content("测工具").clubId(1111).productId(1111);
            MqInfo messageLine = new MqInfo().topic("MessageLine").delay(3).async(0);
            messageModel = messageModel.code(MessageType.WEB_INSIDE_MESSAGE).mqInfo(messageLine).info(sud);
        }
        if (1 == status) {
            WechatTemplateMessage wechatTemplateMessage = new WechatTemplateMessage();
            ArrayList<String> keyWords = new ArrayList<>();
            keyWords.add("我测一下");
            keyWords.add("110");
            keyWords.add("北京天安门广场");
            keyWords.add("2022-10-20");
            wechatTemplateMessage = wechatTemplateMessage.url("https://www.caimei365.com/").templateId("o1nMAIsHAFr0VOu0qEbI1rEFDS14hl40UXbt52I5row")
                    .openId(openId!=null?openId:"o9nzn5wFDZ_WjNo1-SfhqbKNXr4A").appid("wxf3cd4ae0cdd11c36").pagePath("pages/seller/club/list?type=wechat")
                    .keyWords(keyWords).first("你有一个新机构用户，请及时跟进").remark("点击查看详情，了解更多客户信息。");
            MqInfo messageLine = new MqInfo().topic("MessageLine").delay(3).async(0);
            messageModel = messageModel.code(MessageType.WECHAT_TEMPLATE_MESSAGE).info(wechatTemplateMessage).mqInfo(messageLine);
        }
        messageSender.messageSend(messageModel);
        return ResponseJson.success();
    }
}
