package com.caimei365.tools.listener;

import com.caimei365.tools.model.po.IpSavePo;
import com.caimei365.tools.model.po.SearchFrequencyVo;
import com.caimei365.tools.service.CmBehaviorRecordService;
import com.caimei365.tools.service.ProductService;
import com.caimei365.tools.service.impl.IpSaveService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author Administrator
 */
@Slf4j
@Component
@RocketMQMessageListener(
        topic = "IpSave",
        // messageModel = MessageModel.BROADCASTING,//指定为广播消费
        // consumeMode = ConsumeMode.ORDERLY, // 指定消费模式为顺序消费，消费的顺序也和发送顺序一致
        // selectorType = SelectorType.TAG,// 如果我们的生产者指定了Tag，但是消费者的selectorExpression没有设置，即用默认的“*”,那么这个消费者也会消费到
        // selectorExpression = "tag",     // 指定了tag后，发送的消息如果不带tag，将会消费不到
        consumerGroup = "Ip_group")
public class IpSaveListener implements RocketMQListener<String> {

    @Autowired
    private IpSaveService ipSaveService;
    @Resource
    private ProductService productService;
    @Autowired
    private CmBehaviorRecordService recordService;

    @Override
    public void onMessage(String message) {
        log.info("{}收到消息：{}", this.getClass().getSimpleName(), message);
        String[] params = message.split("￥");
        //ip,接口路径，接口参数，注解名
        String ip = params[0];
        String path = params[1];
        String param = params[2];
        String saveName = params[3];
        IpSavePo ipSavePo = new IpSavePo();
        SearchFrequencyVo searchFrequencyVo = new SearchFrequencyVo();
        String productFlag = "";
        String linkageFlag = "";
        searchFrequencyVo.setPath(path);
        if (StringUtils.isNotBlank(param)) {
            ipSavePo.setParams(param);
            if (param.contains("userId") || param.contains("keyword")) {
                String[] split = param.split("&");
                for (String s : split) {
                    if (s.contains("userId") && s.contains("=")) {
                        String[] split1 = s.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                ipSavePo.setUserId(trim);
                            }
                        }
                    }
                    if (s.contains("keyword") && s.contains("=")) {
                        String[] split1 = s.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            try {
                                String va = java.net.URLDecoder.decode(value, "UTF-8");
                                //部分美国端口发送过来的参数需要二次解码(keyword已编码过)
                                if (va.contains("%")) {
                                    va = java.net.URLDecoder.decode(va, "UTF-8");
                                }
                                ipSavePo.setKeyWord(va);
                                searchFrequencyVo.setKeyword(va);
                            } catch (UnsupportedEncodingException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                    if (s.contains("productFlag") && s.contains("=")) {
                        String[] split1 = s.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            try {
                                String va = java.net.URLDecoder.decode(value, "UTF-8");
                                //部分美国端口发送过来的参数需要二次解码(keyword已编码过)
                                if (va.contains("%")) {
                                    va = java.net.URLDecoder.decode(va, "UTF-8");
                                }
                                ipSavePo.setKeyWord(va);
                                productFlag = va;
                            } catch (UnsupportedEncodingException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                    if (s.contains("linkageFlag") && s.contains("=")) {
                        String[] split1 = s.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            try {
                                String va = java.net.URLDecoder.decode(value, "UTF-8");
                                //部分美国端口发送过来的参数需要二次解码(keyword已编码过)
                                if (va.contains("%")) {
                                    va = java.net.URLDecoder.decode(va, "UTF-8");
                                }
                                ipSavePo.setKeyWord(va);
                                linkageFlag = va;
                            } catch (UnsupportedEncodingException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }
            }
        }
        if (StringUtils.isNotBlank(path)) {
            ipSavePo.setRequestUrl(path);
        }
        if (StringUtils.isNotBlank(saveName)) {
            ipSavePo.setPortName(saveName);
        }
        if (StringUtils.isNotBlank(ip)) {
            ipSavePo.setIp(ip);
        }
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        String nowDate = format.format(new Date());
        Long aLong = Long.valueOf(nowDate);
        ipSavePo.setSaveTime(aLong);
        //根据来源统计关键词
        //首页搜索  fromSearch = 1
        log.info("-------------开始统计------------");
        if (null != searchFrequencyVo && StringUtils.isNotEmpty(productFlag) && "1".equals(productFlag) && "/commodity/search/query/product".equals(searchFrequencyVo.getPath())) {
            searchFrequencyVo.setFromSearch(1);
            if (StringUtils.isNotBlank(searchFrequencyVo.getKeyword())) {
                productService.conKeyword(searchFrequencyVo.getFromSearch(), searchFrequencyVo.getPath(), searchFrequencyVo.getKeyword(), linkageFlag);
            }
        }
        //信息中心搜索  fromSearch = 2
        if (null != searchFrequencyVo && "/commodity/search/query/article".equals(searchFrequencyVo.getPath())) {
            searchFrequencyVo.setFromSearch(2);
            if (StringUtils.isNotBlank(searchFrequencyVo.getKeyword())) {
                productService.conKeyword(searchFrequencyVo.getFromSearch(), searchFrequencyVo.getPath(), searchFrequencyVo.getKeyword(), linkageFlag);
            }
        }
        //美业资料搜索 fromSearch=3
        if (null != searchFrequencyVo && "/commodity/product/archive".equals(searchFrequencyVo.getPath())) {
            searchFrequencyVo.setFromSearch(3);
            if (StringUtils.isNotBlank(searchFrequencyVo.getKeyword())) {
                productService.conKeyword(searchFrequencyVo.getFromSearch(), searchFrequencyVo.getPath(), searchFrequencyVo.getKeyword(), linkageFlag);
            }
        }
//        //供应商主页搜索 fromSearch=4 （2022.11.15暂时不考虑该关键词信息来源）
//        if(null !=searchFrequencyVo &&"commodity/search/query/shop".equals(searchFrequencyVo.getPath())){
//            searchFrequencyVo.setFromSearch(4);
//            if(StringUtils.isNotBlank(searchFrequencyVo.getKeyword())){
//                productService.conKeyword(searchFrequencyVo.getFromSearch(),searchFrequencyVo.getPath(),searchFrequencyVo.getKeyword());
//            }
//        }
        //采美百科搜索 fromSearch=5
        if (null != searchFrequencyVo && "/commodity/search/query/baike/keyword".equals(searchFrequencyVo.getPath())) {
            searchFrequencyVo.setFromSearch(4);
            if (StringUtils.isNotBlank(searchFrequencyVo.getKeyword())) {
                productService.conKeyword(searchFrequencyVo.getFromSearch(), searchFrequencyVo.getPath(), searchFrequencyVo.getKeyword(), linkageFlag);
            }
        }
        // 用户行为记录
        if ("/user/record/Statistics".equals(ipSavePo.getRequestUrl().trim())) {
            recordService.insertRecord(ipSavePo);
        }
        ipSaveService.save(ipSavePo);
    }
}
