package com.caimei365.tools.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.tools.mapper.CmBehaviorRecordMapper;
import com.caimei365.tools.model.po.*;
import com.caimei365.tools.service.CmBehaviorRecordService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/11/2
 */
@Slf4j
@Service
public class CmBehaviorRecordServiceImpl implements CmBehaviorRecordService {

    @Autowired
    private CmBehaviorRecordMapper recordMapper;

    private Pattern pattern = Pattern.compile("[a-zA-Z]");

    @Override
    public void insertRecord(IpSavePo ipSavePo) {
        if ("/user/record/Statistics".equals(ipSavePo.getRequestUrl().trim()) && !ipSavePo.getIp().equals("106.55.202.118")) {
            log.info("用户行为记录=================》" + ipSavePo.getIp());
            CmBehaviorRecordPo cmBehaviorRecordPo = new CmBehaviorRecordPo();
            // 赋默认值
            cmBehaviorRecordPo.setPageType(0);
            String region = "";
            String[] splitArray = ipSavePo.getIp().split(",");
            for (String split : splitArray) {
                Matcher matcher = pattern.matcher(split);
                if (!matcher.find()) {
                    cmBehaviorRecordPo.setIP(split);
                }
            }
            Matcher matcher = pattern.matcher(ipSavePo.getIp());
            if (matcher.find()) {
                cmBehaviorRecordPo.setIP(splitArray[1]);
            }
            cmBehaviorRecordPo.setIP(ipSavePo.getIp());
            if (StringUtils.isNotBlank(ipSavePo.getUserId())) {
                cmBehaviorRecordPo.setUserId(ipSavePo.getUserId());
            } else {
                cmBehaviorRecordPo.setUserId("0");
            }
            boolean flag = true;
            if (ipSavePo.getParams().contains("userId") || ipSavePo.getParams().contains("productId")) {
                String[] split = ipSavePo.getParams().split("&");

                for (String s : split) {
                    String str = s.trim();
                    // 页面路径
                    if (str.startsWith("pagePath") && str.contains("=")) {
                        String[] split1 = str.split(str.substring(str.indexOf("="), str.indexOf("=") + 1), 2);
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                String decode = null;
                                try {
                                    decode = java.net.URLDecoder.decode(trim, "UTF-8");
                                    if (decode.contains("%")) {
                                        decode = java.net.URLDecoder.decode(decode, "UTF-8");
                                    }
                                } catch (UnsupportedEncodingException e) {
                                    e.printStackTrace();
                                }
                                cmBehaviorRecordPo.setPagePath(decode);
                            }
                            if (StringUtils.isEmpty(cmBehaviorRecordPo.getPagePath())) {
                                log.info("页面路径pagePath 值为====》" + trim);
                                return;
                            }
                        }
                    }
                    // 页面类型
                    if (str.startsWith("pageType") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                int parseInt = Integer.parseInt(trim);

                                cmBehaviorRecordPo.setPageType(parseInt);
                            }
                        }
                    }
                    // 页面标签
                    if (str.startsWith("pageLabel") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        cmBehaviorRecordPo.setPageLabel("");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {

                                // 接口参数解码
                                String decode = null;
                                try {
                                    decode = java.net.URLDecoder.decode(trim, "UTF-8");
                                    if (decode.contains("%")) {
                                        decode = java.net.URLDecoder.decode(decode, "UTF-8");
                                    }
                                } catch (UnsupportedEncodingException e) {
                                    e.printStackTrace();
                                }
                                cmBehaviorRecordPo.setPageLabel(decode);
                            }
                        }
                    }

                    // 用户行为记录 or roos页面统计
                    if (str.startsWith("behaviorType") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        cmBehaviorRecordPo.setBehaviorType("1");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                if (!trim.equals("null")) {
                                    cmBehaviorRecordPo.setBehaviorType(trim);
                                } else {
                                    cmBehaviorRecordPo.setBehaviorType("0");
                                }

                            }
                        }
                    }
                    // 供应商Id
                    if (str.startsWith("shopId") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        cmBehaviorRecordPo.setShopId(null);
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                if (!trim.equals("null")) {
                                    cmBehaviorRecordPo.setShopId(Integer.parseInt(trim));
                                } else {
                                    cmBehaviorRecordPo.setShopId(null);
                                }
                            }
                        }
                    }
                    // 访问页面类型为供应商主页 且供应商 id 不为空的时候 将页面 pageLabel 改为供应商名称
                    if (null != cmBehaviorRecordPo.getShopId() && 14 == cmBehaviorRecordPo.getPageType()) {
                        // 获取供应商名称
                        String shopName = recordMapper.getShopName(cmBehaviorRecordPo.getShopId());
                        if (StringUtils.isNotBlank(shopName)) {
                            cmBehaviorRecordPo.setPageLabel(shopName);
                        }

                    }
                    // 商品ID
                    if (str.startsWith("productId") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                cmBehaviorRecordPo.setProductId(trim);
                            }
                        }
                    }
                    // 访问客户端 0：网站 1：小程序
                    if (str.startsWith("accessClient") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        cmBehaviorRecordPo.setAccessClient("0");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                cmBehaviorRecordPo.setAccessClient(trim);
                            }
                        }
                    }
                    // 浏览器链接
                    if (str.startsWith("referer") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                // 浏览器链接
                                cmBehaviorRecordPo.setReferer(trim);
                                // 访问来源
                                String source = source(trim);
                                cmBehaviorRecordPo.setAccessSource(source);
                            }
                        }
                    }
                    // 设置删除标记 判定爬虫
                    cmBehaviorRecordPo.setDelFlag("0");
                    // 浏览器userAgent
                    if (str.startsWith("userAgent") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                // 浏览器userAgent
                                cmBehaviorRecordPo.setUserAgent(trim);
                                flag = boolReptiles(trim);
                                if (flag) {
                                    cmBehaviorRecordPo.setDelFlag("1");
                                }
                            }
                        }
                    }
                    // 游客编号
                    if (str.startsWith("touristId") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                log.info("touristId============》" + trim);
                                cmBehaviorRecordPo.setTouristId(trim);
                            }
                        }
                    }
                    if (StringUtils.isEmpty(cmBehaviorRecordPo.getTouristId()) || StringUtils.isBlank(cmBehaviorRecordPo.getTouristId())) {
                        cmBehaviorRecordPo.setTouristId("000000");
                    }
                    // openId
                    if (str.startsWith("openId") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                log.info("openId============》" + trim);
                                cmBehaviorRecordPo.setOpenId(trim);
                            }
                        }
                    }
                    // headUserId
                    if (str.startsWith("headUserId") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                log.info("headUserId============》" + trim);
                                cmBehaviorRecordPo.setHeadUserId(trim);
                            }
                        }
                    }
                    // productArchiveId
                    if (str.startsWith("productArchiveId") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                log.info("productArchiveId============》" + trim);
                                cmBehaviorRecordPo.setProductArchiveId(trim);
                            }
                        }
                    }
                    if (str.startsWith("partnerId") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                log.info("partnerId============》" + trim);
                                cmBehaviorRecordPo.setPartnerId(Integer.valueOf(trim));
                            }
                        }
                    }
                }
                // 处理页面标签为空
                if (StringUtils.isBlank(cmBehaviorRecordPo.getPageLabel()) || StringUtils.isEmpty(cmBehaviorRecordPo.getPageLabel())) {
                    String pageLabel = "";
                    if (6 == cmBehaviorRecordPo.getPageType()) {
                        // 商品详情
                        String productId = "";
                        if ("0".equals(cmBehaviorRecordPo.getAccessClient())) {
                            // 网站访问
                            String[] labelPath = cmBehaviorRecordPo.getPagePath().split("-");
                            if (labelPath[1].contains(".html")) {
                                // http://localhost:8009/product-1017.html
                                productId = labelPath[1].substring(0, labelPath[1].lastIndexOf(".html"));
                            } else {
                                // http://localhost:8009/product-7425-5.html
                                productId = labelPath[1];
                            }
                        } else {
                            // 小程序访问
                            if (cmBehaviorRecordPo.getPagePath().contains("id=")) {
                                if (cmBehaviorRecordPo.getPagePath().contains("&typeId")) {
                                    // /pages/goods/product?id=7425&typeId=5
                                    productId = cmBehaviorRecordPo.getPagePath().substring(cmBehaviorRecordPo.getPagePath().indexOf("id="), cmBehaviorRecordPo.getPagePath().lastIndexOf("&typeId"));
                                } else {
                                    // /pages/goods/product?id=1017
                                    productId = cmBehaviorRecordPo.getPagePath().substring(cmBehaviorRecordPo.getPagePath().indexOf("id=") + 3);
                                }
                            }
                        }
                        try {
                            pageLabel = recordMapper.getProductPageLabel(Integer.parseInt(productId.trim()));
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    } else if (11 == cmBehaviorRecordPo.getPageType()) {
                        String infoId = "";
                        // 文章详情
                        if ("0".equals(cmBehaviorRecordPo.getAccessClient())) {
                            // 网站
                            // https://www.caimei365.com/info/detail-5555-1.html
                            String[] labelPath = cmBehaviorRecordPo.getPagePath().split("-");
                            if (labelPath[1].contains(".html")) {
                                // 暂无该情况
                                infoId = labelPath[1].substring(0, labelPath[1].lastIndexOf(".html"));
                            } else {
                                infoId = labelPath[1];
                            }
                        }
                        try {
                            pageLabel = recordMapper.getInfoPageLabel(Integer.parseInt(infoId.trim()));
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    } else if (12 == cmBehaviorRecordPo.getPageType()) {
                        String baikeId = "";
                        // 采美百科
                        if ("0".equals(cmBehaviorRecordPo.getAccessClient())) {
                            // 网站
                            // https://www.caimei365.com/encyclopedia/instrument-10.html
                            String[] labelPath = cmBehaviorRecordPo.getPagePath().split("-");
                            if (labelPath[1].contains(".html")) {
                                baikeId = labelPath[1].substring(0, labelPath[1].lastIndexOf(".html"));
                            } else {
                                // 暂无该情况
                                baikeId = labelPath[1];
                            }
                        }
                        try {
                            pageLabel = recordMapper.getBaiKePageLabel(Integer.parseInt(baikeId.trim()));
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                    if (StringUtils.isBlank(pageLabel) || StringUtils.isEmpty(pageLabel)) {
                        // 为空查询页面标签、或者使用初始默认数据
                        pageLabel = recordMapper.getPageLabel(cmBehaviorRecordPo.getPageType());
                    } else {
                        pageLabel = " ";
                    }
                    cmBehaviorRecordPo.setPageLabel(pageLabel);
                }
            }
            Date date = new Date();
            cmBehaviorRecordPo.setAccessTime(date);
            cmBehaviorRecordPo.setAccessDuration("5000");
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
            String format = simpleDateFormat.format(date);
            cmBehaviorRecordPo.setAccessDate(format);
            cmBehaviorRecordPo.setIsReckon("0");
            // 确定ip所属地

            try {
                region = recordIp(ipSavePo.getIp());
            } catch (IOException e) {
                e.printStackTrace();
            }
            cmBehaviorRecordPo.setRegion(region);
            // if (flag) {
            recordMapper.insertRecord(cmBehaviorRecordPo);

            int recordId = cmBehaviorRecordPo.getRecordID();
            if (cmBehaviorRecordPo.getShopId() != null) {
                if (14 != cmBehaviorRecordPo.getPageType()) {
                    if (StringUtils.isNotBlank(cmBehaviorRecordPo.getPageLabel())) {
                        List<String> strList = new ArrayList<>();
                        // 页面标签在含有多个，包含 ， 是可分割，进行分割作为关键词
                        if (cmBehaviorRecordPo.getPageLabel().contains(",")) {
                            String[] split = cmBehaviorRecordPo.getPageLabel().split(",");
                            for (String s : split) {
                                strList.add(s);
                            }
                        } else {
                            strList.add(cmBehaviorRecordPo.getPageLabel());
                        }
                        // 供应商id
                        List<Integer> shopIds = recordMapper.shopIds(strList);
                        if (shopIds.size() > 0) {
                            for (Integer shopId : shopIds) {
                                recordMapper.insertShopId(recordId, shopId);
                            }
                        }
                    }
                } else {
                    Integer id = recordMapper.selShopId(cmBehaviorRecordPo.getShopId());
                    if (id != null) {
                        recordMapper.insertShopId(recordId, cmBehaviorRecordPo.getShopId());
                    }
                }
            }
            //}
        }
    }

    @Override
    public void updateBehaviorTime() {
        log.info("计算实际访问时长==================》");
        long t1 = 0;
        long t2 = 0;
        Long duration = 0L;
        CmBehaviorRecordPo cmBehaviorRecordPo = new CmBehaviorRecordPo();
        CmBehaviorRecordPo cmBehaviorRecord = new CmBehaviorRecordPo();
        Date date = new Date();
        // 获取当前时间日期
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String format = simpleDateFormat.format(new Date(date.getTime() - 600000));
        cmBehaviorRecordPo.setAccessDate(format);
        // 当天访问系统ip和userID
        List<CmBehaviorRecordPo> recordPos = recordMapper.toDateRecode(cmBehaviorRecordPo);
        for (CmBehaviorRecordPo behaviorRecordPo : recordPos) {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            // 当前时间
            String timeEnd = dateFormat.format(date);
            //  一小时前时间
            Calendar c = Calendar.getInstance();
            c.set(Calendar.HOUR_OF_DAY, (c.get(Calendar.HOUR_OF_DAY) - 1));
            String timeStart = dateFormat.format(c.getTime());

            behaviorRecordPo.setStartTime(timeStart);
            behaviorRecordPo.setEndTime(timeEnd);
            // 获取一个小时之内的数据
            List<CmBehaviorRecordPo> recordPoList = recordMapper.toDateIPTimeRecode(behaviorRecordPo);
            if (recordPoList.size() <= 0) {
                continue;
            }
            for (int i = 0; i < recordPoList.size(); i++) {
                if (i != recordPoList.size() - 1) {
                    t1 = recordPoList.get(i).getAccessTime().getTime();
                    t2 = recordPoList.get(i + 1).getAccessTime().getTime();
                    duration = t2 - t1;
                    // 修改实际访问时长
                    cmBehaviorRecord.setRecordID(recordPoList.get(i).getRecordID());
                    cmBehaviorRecord.setAccessDuration(duration.toString());
                    cmBehaviorRecord.setIsReckon("1");
                    recordMapper.updateDuration(cmBehaviorRecord);
                }
            }
        }
    }

    @Override
    public void setIpAddress(Integer startId, Integer endId) throws IOException {
        List<CmUserPo> list = recordMapper.findList(startId, endId);
        String region = "";
        for (CmUserPo user : list) {
            try {
                // 获取ip所在地
                region = recordIp(user.getRegisterIP());
                user.setIpAddress(region);
                recordMapper.updateOldData(user);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void setProductCostProportional() {
        List<CmSkuPo> skuList = recordMapper.skuList();
        skuList.forEach(sku -> {
            if (null != sku.getPrice() && 0 != sku.getPrice()) {
                // 供应商比例
                double shopPercent = (sku.getCostPrice() / sku.getPrice() * 100);
                double rint = Math.rint(shopPercent);
                // 分账组织佣金比例
                double organizePercent = 0d;
                // 分账采美佣金比例
                double cmPercent = 100d - rint;
                sku.setShopPercent(rint);
                sku.setOrganizePercent(organizePercent);
                sku.setCmPercent(cmPercent);
                // 成本类型 -> 比例成本
                sku.setCostCheckFlag(2);
                recordMapper.updateSku(sku);
            }
        });
    }

    @Override
    public void updateBehaviorUser() {
        log.info("====合并IP====");
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        Date time = calendar.getTime();
        String format = dateFormat.format(time);
        Integer num = 0;
        // 小程序 合并数据使用openId
        List<String> openIdList = recordMapper.getRecodeOpenId(format);
        if (null != openIdList && openIdList.size() > 0) {
            // 程序中每个openid有多个用户
            List<CmBehaviorRecordPo> cuxOpen = recordMapper.getCuxOpen(format);
            // 查询小程序数据
            List<CmBehaviorRecordPo> behaviorList = recordMapper.getRecordOpenIdInfo(format, openIdList);
            for (String openId : openIdList) {
                // 用户个数
                CmBehaviorRecordPo recordPo = cuxOpen.stream().filter(open -> openId.equals(open.getOpenId())).collect(Collectors.toList()).get(0);
                num = null == recordPo.getCount() ? 0 : recordPo.getCount();
                // 获取该openId下访问的用户id
                List<Integer> userIdAttribute = recordMapper.getUserIdAttribute(format, openId, null);
                boolean userBean = userIdAttribute.contains(0);

                // 该openId下只有两个身份 游客-机构、游客-供应商
                if (num == 2 && userBean) {
                    // 将游客数据中的ip和userId修改为机构/供应商数据
                    for (CmBehaviorRecordPo cmBehaviorRecordPo : behaviorList) {
                        if (openId.equals(cmBehaviorRecordPo.getOpenId())) {
                            cmBehaviorRecordPo.setIP(cmBehaviorRecordPo.getIP());
                            cmBehaviorRecordPo.setUserId(cmBehaviorRecordPo.getUserId());
                            cmBehaviorRecordPo.setOpenId(openId);
                            cmBehaviorRecordPo.setAccessDate(format);
                            recordMapper.updateBehavior(cmBehaviorRecordPo);
                        }
                    }
                }
            }
        }
        // 网站 合并按游客Id 合并数据
        List<String> recodeTouristId = recordMapper.getRecodeTouristId(format);
        if (null != recodeTouristId && recodeTouristId.size() > 0) {
            // 网站中每个touristId有多个用户
            List<CmBehaviorRecordPo> cuxTourist = recordMapper.getCuxTourist(format);
            // 游客对应用户信息
            List<CmBehaviorTouristInfoPo> touristInfo = recordMapper.getTouristInfo(format, recodeTouristId);
            if (null != touristInfo) {
                for (String touristId : recodeTouristId) {
                    // 用户个数
                    CmBehaviorRecordPo recordPo = cuxTourist.stream().filter(tourist -> tourist.getTouristId().equals(touristId)).collect(Collectors.toList()).get(0);
                    num = null == recordPo.getCount() ? 0 : recordPo.getCount();
                    // 获取该openId下访问的用户id
                    List<Integer> userIdAttribute = recordMapper.getUserIdAttribute(format, null, touristId);
                    boolean userBean = userIdAttribute.contains(0);
                    // 该openId下只有两个身份 游客-机构、游客-供应商
                    if (num == 2 && userBean) {
                        for (CmBehaviorTouristInfoPo touristInfoPo : touristInfo) {
                            if (touristId.equals(touristInfoPo.getTouristId())) {
                                CmBehaviorRecordPo cmBehaviorRecordPo = new CmBehaviorRecordPo();
                                cmBehaviorRecordPo.setIP(recordMapper.getTouristIdByIp(touristId, format));
                                cmBehaviorRecordPo.setUserId(touristInfoPo.getUserId().toString());
                                cmBehaviorRecordPo.setTouristId(touristId);
                                cmBehaviorRecordPo.setAccessDate(format);
                                //根据游客信息合并用户浏览记录
                                recordMapper.updateBehaviorByTouristId(cmBehaviorRecordPo);
                            }
                        }
                    }
                }
            }
        }
        // 将当天访问ip和注册用户ip关联
        List<CmBehaviorRecordPo> userAndBehavior = recordMapper.getUserAndBehavior(format);
        if (null != userAndBehavior && userAndBehavior.size() > 0) {
            // 将相同的合并为该用户的访问
            userAndBehavior.forEach(behavior -> {
                recordMapper.updateBehaviorTourist(Integer.parseInt(behavior.getUserId()), behavior.getRecordID());
            });
        }
    }

    /**
     * 昨日用户行为列表数据生成
     */
    @Override
    public void insertBehaviorUser() {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        Date time = calendar.getTime();
        String format = dateFormat.format(time);
        // 获取昨天数据
        List<CmBehaviorRecordPo> yesterdayList = recordMapper.selYesterdayList(format);
        // 详细数据
        List<CmBehaviorRecordPo> selDataList = recordMapper.selDataList(format);
        // 获取供应商关联所有标签
        List<String> keywords = recordMapper.shopKeyword();
        for (CmBehaviorRecordPo data : yesterdayList) {
            ArrayList<String> item = new ArrayList();
            ArrayList<String> shopNames = new ArrayList();
            // 当ip过长，有多个ip时已第一个IP为准
            if (data.getIP().contains(",")) {
                String[] splitArray = data.getIP().split(",");
                for (String split : splitArray) {
                    Matcher matcher = pattern.matcher(split);
                    if (!matcher.find()) {
                        data.setIP(split);
                    }
                }
                Matcher matcher = pattern.matcher(data.getIP());
                if (matcher.find()) {
                    data.setIP(splitArray[1]);
                }
            }
            if (data.getIP().length() > 20) {
                data.setDelFlag("1");
            }
            int num = 0;
            for (CmBehaviorRecordPo recordPo : selDataList) {
                if (recordPo.getIP().equals(data.getIP()) && recordPo.getUserId().equals(data.getUserId())) {
                    // 标签信息 对多存储10个
                    if (num < 10) {
                        if (StringUtils.isNotBlank(recordPo.getPageLabel())) {
                            if (recordPo.getPageLabel().contains(",")) {
                                String[] split = recordPo.getPageLabel().split(",");
                                for (String s : split) {
                                    if (keywords.contains(s)) {
                                        if (!item.contains(s)) {
                                            item.add(s);
                                            num++;
                                        }
                                    }
                                }
                            } else {
                                if (keywords.contains(recordPo.getPageLabel())) {
                                    if (!item.contains(recordPo.getPageLabel())) {
                                        item.add(recordPo.getPageLabel());
                                        num++;
                                    }
                                }
                            }

                        }
                    }
                    if (StringUtils.isNotBlank(recordPo.getRelevanceShop()) && !shopNames.contains(recordPo.getRelevanceShop())) {
                        shopNames.add(recordPo.getRelevanceShop());
                    }
                }
            }
            data.setLabel(StringUtils.strip(item.toString(), "[]"));
            data.setRelevanceShop(StringUtils.strip(shopNames.toString(), "[]"));
            // 获取该用户该天页面标签
            List<String> behaviorPageLabel = recordMapper.getBehaviorPageLabel(data.getIP(), data.getAccessDate(), data.getUserId());
            data.setPageLabels(StringUtils.strip(behaviorPageLabel.toString(), "[]"));
            // 获取该用户该天页面类型
            List<Integer> behaviorPageType = recordMapper.getBehaviorPageType(data.getIP(), data.getAccessDate(), data.getUserId());
            data.setPageTypes(StringUtils.strip(behaviorPageType.toString(), "[]"));
            if (StringUtils.isBlank(data.getPageLabel()) || StringUtils.isEmpty(data.getPageLabel())) {
                data.setPageLabels(" ");
            }
            // 数据插入
            recordMapper.insertYesterdayList(data);
        }
        log.info("====昨日用户行为列表数据已生成====");
    }

    /**
     * 生成用户行为今日列表数据 数据一小时覆盖
     */
    @Override
    public void insertTodayData() {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String format = dateFormat.format(new Date());


        // 今日用户行为数据
        List<CmBehaviorRecordPo> todayData = recordMapper.selTodayData(format);
        // 清除今日用户行为记录
        recordMapper.delTodayData();
        // 生成今日用户行为记录
        int id = 1;
        // 所有用户供应商数据
        List<CmBehaviorRecordPo> selDataList = recordMapper.selDataList(format);
        List<String> keywords = recordMapper.shopKeyword();
        for (CmBehaviorRecordPo data : todayData) {
            ArrayList<String> item = new ArrayList();
            ArrayList<String> shopNames = new ArrayList();
            // 当ip过长，有多个ip时已第一个IP为准
            if (data.getIP().contains(",")) {
                String[] splitArray = data.getIP().split(",");

                for (String split : splitArray) {
                    Matcher matcher = pattern.matcher(split);
                    if (!matcher.find()) {
                        data.setIP(split);
                    }
                }
                Matcher matcher = pattern.matcher(data.getIP());
                if (matcher.find()) {
                    data.setIP(splitArray[1]);
                }
            }
            if (data.getIP().length() > 20) {
                data.setDelFlag("1");
            }
            int num = 0;
            data.setRecordID(id);
            id++;
            for (CmBehaviorRecordPo recordPo : selDataList) {
                if (recordPo.getIP().equals(data.getIP()) && recordPo.getUserId().equals(data.getUserId())) {
                    // 标签信息
                    if (num < 10) {
                        if (StringUtils.isNotBlank(recordPo.getPageLabel())) {
                            if (recordPo.getPageLabel().contains(",")) {
                                String[] split = recordPo.getPageLabel().split(",");
                                for (String s : split) {
                                    if (keywords.contains(s)) {
                                        if (!item.contains(s)) {
                                            item.add(s);
                                            num++;
                                        }
                                    }
                                }
                            } else {
                                if (keywords.contains(recordPo.getPageLabel())) {
                                    if (!item.contains(recordPo.getPageLabel())) {
                                        item.add(recordPo.getPageLabel());
                                        num++;
                                    }
                                }
                            }

                        }
                    }
                    // 供应商
                    if (StringUtils.isNotBlank(recordPo.getRelevanceShop()) && !shopNames.contains(recordPo.getRelevanceShop())) {
                        shopNames.add(recordPo.getRelevanceShop());
                    }
                }
            }

            data.setLabel(StringUtils.strip(item.toString(), "[]"));
            data.setRelevanceShop(StringUtils.strip(shopNames.toString(), "[]"));
            // 获取该用户该天页面标签
            List<String> behaviorPageLabel = recordMapper.getBehaviorPageLabel(data.getIP(), data.getAccessDate(), data.getUserId());
            data.setPageLabels(StringUtils.strip(behaviorPageLabel.toString(), "[]"));
            // 获取该用户该天页面类型
            List<Integer> behaviorPageType = recordMapper.getBehaviorPageType(data.getIP(), data.getAccessDate(), data.getUserId());
            data.setPageTypes(StringUtils.strip(behaviorPageType.toString(), "[]"));
            if (StringUtils.isBlank(data.getPageLabel()) || StringUtils.isEmpty(data.getPageLabel())) {
                data.setPageLabels(" ");
            }
            recordMapper.inBehaviorToday(data);
            log.info("====今日用户行为列表数据已生成====");
        }
    }

    @Override
    public void updateClubAddress(Integer startId, Integer endId) {
        List<CmBehaviorRecordPo> recordPoList = recordMapper.selRegisterIP(startId, endId);
        if (recordPoList.size() > 0) {
            recordPoList.forEach(r -> {
                if (StringUtils.isNotBlank(r.getRegisterIp()) && (null == r.getProvinceId() && null == r.getCityId())) {
                    log.info("个人机构地址初始化=====》" + r.getClubId());
                    try {
                        Map<String, String> map = recordAddress(r.getRegisterIp());
                        if (!map.isEmpty()) {
                            // 省份
                            String regionPro = map.get("regionPro");
                            Integer provinceId = recordMapper.selProvince(regionPro);
                            // 市
                            String regionCity = map.get("regionCity");
                            AddressPo addressPo = recordMapper.selCity(regionCity);

                            Integer cityId = null;
                            if (null != provinceId) {
                                if (null != addressPo) {
                                    if (null != addressPo.getProvinceId()) {
                                        if (addressPo.getProvinceId().equals(provinceId)) {
                                            // 省份id与地址表的省份地址对应 取地址表地址 机构给予 省、市地址
                                            recordMapper.upClubAddress(r.getClubId(), provinceId, addressPo.getCityId());
                                        } else {
                                            // 省份id与地址表的省份地址不对应 取地址表地址
                                            provinceId = addressPo.getProvinceId();
                                            recordMapper.upClubAddress(r.getClubId(), provinceId, addressPo.getCityId());
                                        }
                                    } else {
                                        // 地址表没有省份id 机构给予省份地址Id
                                        recordMapper.upClubAddress(r.getClubId(), provinceId, cityId);
                                    }
                                } else {
                                    // 省份不为空 市为空，机构给予省份地址Id
                                    recordMapper.upClubAddress(r.getClubId(), provinceId, cityId);
                                }
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            });
        }
    }

    /**
     * 页面统计
     */
    @Override
    public void pageStatistics() {
        log.info("页面统计数据");
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Calendar instance = Calendar.getInstance();
        instance.setTime(new Date());
        instance.add(Calendar.DAY_OF_MONTH, -1);
        Date time = instance.getTime();
        String format = dateFormat.format(time);
        log.info("dateTime========" + format);
        // 统计供应商
        List<Integer> shopIds = recordMapper.getShopId();
        shopIds.forEach(s -> {
            log.info("统计供应商数据");
            // 检索供应商统计信息
            CmBehaviorRecordPo behavior = recordMapper.getBehaviorList(format, 14, s.toString());
            CmPageShopPo cmPageShopPo = new CmPageShopPo();
            cmPageShopPo.setShopId(s);
            cmPageShopPo.setAccessNumber(Integer.parseInt(behavior.getNumber()));
            cmPageShopPo.setShopLink(behavior.getPagePath());
            cmPageShopPo.setAccessDuration(behavior.getAccessDuration());
            cmPageShopPo.setAccessDate(format);
            // 加入统计供应商数据
            recordMapper.insertPageShop(cmPageShopPo);
            log.info("统计供应商商品数据");
            // 需要统计的供应商商品
            List<Integer> productIdIdList = recordMapper.getProductId(s);
            for (Integer productId : productIdIdList) {
                behavior = recordMapper.getBehaviorList(format, 6, productId.toString());
                CmPageShopProductPo productPo = new CmPageShopProductPo();
                productPo.setShopId(s);
                productPo.setProductId(productId);
                productPo.setLink(behavior.getPagePath());
                productPo.setAccessNumber(Integer.parseInt(behavior.getNumber()));
                productPo.setAccessDuration(behavior.getAccessDuration());
                productPo.setAccessDate(format);
                // 加入统计供应商商品数据
                recordMapper.insertPageProduct(productPo);
            }
            // 需要统计的供应商文章
            List<Integer> infoIdList = recordMapper.getShopInfoId(s);
            for (Integer infoId : infoIdList) {
                behavior = recordMapper.getBehaviorList(format, 11, infoId.toString());
                CmPageShopInfoPo infoPo = new CmPageShopInfoPo();
                infoPo.setShopId(s);
                infoPo.setInfoId(infoId);
                infoPo.setLink(behavior.getPagePath());
                infoPo.setAccessNumber(Integer.parseInt(behavior.getNumber()));
                infoPo.setAccessDuration(behavior.getAccessDuration());
                infoPo.setAccessDate(format);
                // 添加供应商文章统计数据
                recordMapper.insertPageInfo(infoPo);
            }
            // 需要统计的供应商搜索词
            List<SearchFrequencyVo> keywordList = recordMapper.getShopKeywordId(s);
            for (SearchFrequencyVo keyword : keywordList) {
                behavior = recordMapper.getBehaviorList(format, 8, keyword.getKeyword());
                CmPageShopKeywordPo keywordPo = new CmPageShopKeywordPo();
                keywordPo.setShopId(s);
                keywordPo.setSearchId(keyword.getId());
                keywordPo.setLink(behavior.getPagePath());
                keywordPo.setAccessNumber(Integer.parseInt(behavior.getNumber()));
                keywordPo.setAccessDuration(behavior.getAccessDuration());
                keywordPo.setAccessDate(format);
                // 添加供应商搜索词统计数据
                recordMapper.insertPageKeyword(keywordPo);
            }
        });
        log.info("页面统计结束");
    }

    /**
     * 机构需求标签
     *
     * @param startTime
     * @param endTime
     */
    @Override
    public void clubDemand(String startTime, String endTime) {
        log.info("机构需求标签====统计开始");
        // 获取机构Id
        List<Integer> clubIdList = recordMapper.getClubId();
        // 动态标签
        List<String> trendsKeywordList = recordMapper.getTrendsKeyword();
        // 静态标签
        List<String> stateKeywordList = recordMapper.getStateKeyword();
        // 机构动态标签
        List<CmClubLabelPo> clubTrendsKeywordList = recordMapper.getClubTrendsKeyword();
        // 机构静态标签
        List<CmClubLabelPo> clubStateKeywordList = recordMapper.getClubStateKeyword();
        if (null != clubIdList && clubIdList.size() > 0) {
            for (Integer clubId : clubIdList) {
                // 标签
                List<String> demandList = new ArrayList<>();
                // 出现次数
                Map<String, Integer> map = new HashMap<>();
                // 对应机构Id
                Map<String, Integer> clubIdMap = new HashMap<>();
                // 咨询记录
                List<CmClubLabelPo> clubLabel = recordMapper.getClubLabel(clubId, startTime, endTime);
                clubLabel.removeIf(Objects::nonNull);
                if (null != clubLabel && clubLabel.size() > 0) {
                    // 动态标签
                    List<CmClubLabelPo> trendList = clubLabel.stream().filter(label -> StringUtils.isNotBlank(label.getTrendsKeyword())).collect(Collectors.toList());
                    List<String> trendsKeywordCollect = new ArrayList<>();
                    if (null != trendList && trendList.size() > 0) {
                        trendsKeywordCollect = trendList.stream().map(CmClubLabelPo::getTrendsKeyword).collect(Collectors.toList());
                    }
                    if (null != trendsKeywordCollect && trendsKeywordCollect.size() > 0) {
                        for (String trendsKeyword : trendsKeywordCollect) {
                            if (trendsKeyword.contains(",")) {
                                String[] split = trendsKeyword.split(",");
                                for (String s : split) {
                                    // 第一次加入， 初始出现系数为 1
                                    if (StringUtils.isNotBlank(s) && StringUtils.isNotEmpty(s)) {
                                        if (!demandList.contains(s)) {
                                            demandList.add(s);
                                            map.put(s, 1);
                                            clubIdMap.put(s, clubId);
                                        } else {
                                            // 记录过，出现次数在原来基础上 + 1
                                            Integer number = map.get(s) + 1;
                                            map.put(s, number);
                                        }
                                    }
                                }
                            } else {
                                // 第一次加入， 初始出现系数为 1
                                if (StringUtils.isNotBlank(trendsKeyword) && StringUtils.isNotEmpty(trendsKeyword)) {
                                    if (!demandList.contains(trendsKeyword)) {
                                        demandList.add(trendsKeyword);
                                        map.put(trendsKeyword, 1);
                                        clubIdMap.put(trendsKeyword, clubId);
                                    } else {
                                        // 记录过，出现次数在原来基础上 + 1
                                        Integer number = map.get(trendsKeyword) + 1;
                                        map.put(trendsKeyword, number);
                                    }
                                }
                            }
                        }
                        // 保存动态标签
                        List<String> collect = new ArrayList<>();
                        if (null != clubTrendsKeywordList && clubTrendsKeywordList.size() > 0) {
                            collect = clubTrendsKeywordList.stream().filter(tr -> tr.getClubId().equals(clubId)).map(CmClubLabelPo::getLabel).collect(Collectors.toList());
                        }
                        for (String demand : demandList) {
                            if (!collect.contains(demand)) {
                                // 不存在 添加进机构标签
                                CmClubLabelPo cmClubLabelPo = new CmClubLabelPo();
                                cmClubLabelPo.setLabel(demand);
                                cmClubLabelPo.setClubId(clubIdMap.get(demand));
                                cmClubLabelPo.setDynamicStatus(0);
                                cmClubLabelPo.setAppearNumber(map.get(demand));
                                recordMapper.insertClubLabel(cmClubLabelPo);
                                collect.add(demand);
                                clubTrendsKeywordList.add(cmClubLabelPo);
                            } else {
                                // 存在时在原有基础累加
                                CmClubLabelPo cmClubLabelPo = clubTrendsKeywordList.stream().filter(trend -> trend.getLabel().equals(demand)).collect(Collectors.toList()).get(0);
                                cmClubLabelPo.setAppearNumber(cmClubLabelPo.getAppearNumber() + 1);
                                recordMapper.updateClubLabel(cmClubLabelPo);
                            }
                        }
                    }
                    // 标签
                    demandList = new ArrayList<>();
                    // 出现次数
                    map = new HashMap<>();
                    // 对应机构Id
                    clubIdMap = new HashMap<>();
                    // 静态标签
                    List<CmClubLabelPo> stateList = clubLabel.stream().filter(label -> StringUtils.isNotBlank(label.getStateKeyword())).collect(Collectors.toList());
                    List<String> stateKeywordCollect = new ArrayList<>();
                    if (null != stateList && stateList.size() > 0) {
                        stateKeywordCollect = stateList.stream().map(CmClubLabelPo::getStateKeyword).collect(Collectors.toList());
                    }
                    if (null != stateKeywordCollect && stateKeywordCollect.size() > 0) {
                        for (String stateKeyword : stateKeywordCollect) {
                            if (stateKeyword.contains(",")) {
                                String[] split = stateKeyword.split(",");
                                for (String s : split) {
                                    // 第一次加入， 初始出现系数为 1
                                    if (StringUtils.isNotBlank(s) && StringUtils.isNotEmpty(s)) {
                                        if (StringUtils.isNotEmpty(s) && !demandList.contains(s)) {
                                            demandList.add(s);
                                            map.put(s, 1);
                                            clubIdMap.put(s, clubId);
                                        } else {
                                            // 记录过，出现次数在原来基础上 + 1
                                            Integer number = map.get(s) + 1;
                                            map.put(s, number);
                                        }
                                    }
                                }
                            } else {
                                // 第一次加入， 初始出现系数为 1
                                if (StringUtils.isNotBlank(stateKeyword) && StringUtils.isNotEmpty(stateKeyword)) {
                                    if (!demandList.contains(stateKeyword)) {
                                        demandList.add(stateKeyword);
                                        map.put(stateKeyword, 1);
                                        clubIdMap.put(stateKeyword, clubId);
                                    } else {
                                        // 记录过，出现次数在原来基础上 + 1
                                        Integer number = map.get(stateKeyword) + 1;
                                        map.put(stateKeyword, number);
                                    }
                                }
                            }
                        }
                        List<String> collect = new ArrayList<>();
                        if (null != clubStateKeywordList && clubStateKeywordList.size() > 0) {
                            collect = clubStateKeywordList.stream().filter(st -> st.getClubId().equals(clubId)).map(CmClubLabelPo::getLabel).collect(Collectors.toList());
                        }
                        // 保存静态标签
                        for (String demand : demandList) {
                            if (!collect.contains(demand)) {
                                CmClubLabelPo cmClubLabelPo = new CmClubLabelPo();
                                cmClubLabelPo.setLabel(demand);
                                cmClubLabelPo.setClubId(clubIdMap.get(demand));
                                cmClubLabelPo.setDynamicStatus(1);
                                cmClubLabelPo.setAppearNumber(map.get(demand));
                                recordMapper.insertClubLabel(cmClubLabelPo);
                                collect.add(demand);
                                clubStateKeywordList.add(cmClubLabelPo);
                            } else {
                                // 存在时在原有基础累加
                                CmClubLabelPo cmClubLabelPo = clubStateKeywordList.stream().filter(trend -> trend.getLabel().equals(demand)).collect(Collectors.toList()).get(0);
                                cmClubLabelPo.setAppearNumber(cmClubLabelPo.getAppearNumber() + 1);
                                recordMapper.updateClubLabel(cmClubLabelPo);
                            }
                        }
                    }
                }
                // 标签
                demandList = new ArrayList<>();
                // 出现次数
                map = new HashMap<>();
                // 对应机构Id
                clubIdMap = new HashMap<>();
                // 标签类型
                Map<String, Integer> pageTypeMap = new HashMap<>();
                // 访问记录
                List<CmBehaviorRecordPo> behaviorRecordList = recordMapper.getClubBehavior(clubId, startTime, endTime);
                log.info("behaviorRecordList====" + behaviorRecordList);
                if (null != behaviorRecordList && behaviorRecordList.size() > 0) {
                    for (CmBehaviorRecordPo behaviorRecord : behaviorRecordList) {
                        if (behaviorRecord.getPageLabel().contains(",")) {
                            String[] split = behaviorRecord.getPageLabel().split(",");
                            for (String s : split) {
                                if (StringUtils.isNotBlank(s) && StringUtils.isNotEmpty(s)) {
                                    if (!demandList.contains(s)) {
                                        demandList.add(s);
                                        map.put(s, Integer.parseInt(behaviorRecord.getNumber()));
                                        clubIdMap.put(s, clubId);
                                        pageTypeMap.put(s, behaviorRecord.getPageType());
                                    } else {
                                        // 记录过，出现次数在原来基础上 + 1
                                        Integer number = map.get(s) + Integer.parseInt(behaviorRecord.getNumber());
                                        map.put(s, number);
                                    }
                                }
                            }
                        } else {
                            if (StringUtils.isNotBlank(behaviorRecord.getPageLabel()) && StringUtils.isNotEmpty(behaviorRecord.getPageLabel())) {
                                if (!demandList.contains(behaviorRecord.getPageLabel())) {
                                    demandList.add(behaviorRecord.getPageLabel());
                                    map.put(behaviorRecord.getPageLabel(), Integer.parseInt(behaviorRecord.getNumber()));
                                    clubIdMap.put(behaviorRecord.getPageLabel(), clubId);
                                    pageTypeMap.put(behaviorRecord.getPageLabel(), behaviorRecord.getPageType());
                                } else {
                                    // 记录过，出现次数在原来基础上 + 1
                                    Integer number = map.get(behaviorRecord.getPageLabel()) + 1;
                                    map.put(behaviorRecord.getPageLabel(), number);
                                }
                            }
                        }
                    }
                    List<String> trendsCollect = new ArrayList<>();
                    List<String> stateCollect = new ArrayList<>();
                    if (null != clubTrendsKeywordList && clubTrendsKeywordList.size() > 0) {
                        trendsCollect = clubTrendsKeywordList.stream().filter(tr -> tr.getClubId().equals(clubId)).map(CmClubLabelPo::getLabel).collect(Collectors.toList());
                        stateCollect = clubStateKeywordList.stream().filter(st -> st.getClubId().equals(clubId)).map(CmClubLabelPo::getLabel).collect(Collectors.toList());
                    }
                    // 保存访问标签
                    log.info("demandList==访问标签==" + demandList);
                    for (String demand : demandList) {
                        if (!trendsCollect.contains(demand) && !stateCollect.contains(demand)) {
                            CmClubLabelPo cmClubLabelPo = new CmClubLabelPo();
                            cmClubLabelPo.setLabel(demand);
                            cmClubLabelPo.setClubId(clubIdMap.get(demand));
                            cmClubLabelPo.setAppearNumber(map.get(demand));
                            cmClubLabelPo.setPageType(pageTypeMap.get(demand));
                            if (trendsKeywordList.contains(demand)) {
                                cmClubLabelPo.setDynamicStatus(0);
                                recordMapper.insertClubLabel(cmClubLabelPo);
                                trendsCollect.add(demand);
                                clubTrendsKeywordList.add(cmClubLabelPo);
                            } else if (stateKeywordList.contains(demand)) {
                                cmClubLabelPo.setDynamicStatus(1);
                                recordMapper.insertClubLabel(cmClubLabelPo);
                                stateCollect.add(demand);
                                clubStateKeywordList.add(cmClubLabelPo);
                            }
                        } else {
                            // 存在时在原有基础累加
                            if (trendsCollect.contains(demand)) {
                                CmClubLabelPo cmClubLabelPo = clubTrendsKeywordList.stream().filter(trend -> trend.getLabel().equals(demand)).collect(Collectors.toList()).get(0);
                                cmClubLabelPo.setAppearNumber(cmClubLabelPo.getAppearNumber() + 1);
                                recordMapper.updateClubLabel(cmClubLabelPo);
                            }
                            if (stateCollect.contains(demand)) {
                                CmClubLabelPo cmClubLabelPo = clubStateKeywordList.stream().filter(trend -> trend.getLabel().equals(demand)).collect(Collectors.toList()).get(0);
                                cmClubLabelPo.setAppearNumber(cmClubLabelPo.getAppearNumber() + 1);
                                recordMapper.updateClubLabel(cmClubLabelPo);
                            }
                        }
                    }
                }
            }
        }
        log.info("机构需求标签====统计结束");
    }


    // 获取IP对应地址 ---- 太平洋
    public static String recordIp(String ip) throws IOException {
        URL url = null;
        HttpURLConnection connection = null;
        String encoding = "gbk";
        String text = "";
        String line = "";
        String region = "";
        String urlStr = "http://whois.pconline.com.cn/ipJson.jsp?ip=" + ip + "&json=true";
        try {
            url = new URL(urlStr);
            connection = (HttpURLConnection) url.openConnection();    //新建链接实例
            connection.setConnectTimeout(20000);    //设置链接超时时间，单位毫秒
            connection.setReadTimeout(20000);        //设置读取数据超时时间，单位毫秒
            connection.setDoOutput(true);        //是否打开输出流true|false
            connection.setDoInput(true);        //是否打开输入流true|false
            connection.setRequestMethod("GET");        // 提交方式get|post
            connection.setUseCaches(false);        // 是否加入缓存true|false
            connection.connect();    //打开链接端口

            BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream(), encoding));    // 往对端写完数据对端服务器返回数据。以BufferedReader流来读取

            while ((line = reader.readLine()) != null) {
                text += line + "\n";
            }
            reader.close();

            JSONObject jsonObject = JSONObject.parseObject(text);
            region = jsonObject.get("addr").toString();
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } finally {
            if (connection != null) {
                connection.disconnect();    //关闭连接
            }
        }
        return region;
    }

    // 获取IP对应地址 ---- 太平洋
    public static Map<String, String> recordAddress(String ip) throws IOException {
        URL url = null;
        HttpURLConnection connection = null;
        String encoding = "gbk";
        String text = "";
        String line = "";
        String regionPro = "";
        String regionCity = "";
        Map<String, String> map = new HashMap<>();
        String urlStr = "http://whois.pconline.com.cn/ipJson.jsp?ip=" + ip + "&json=true";
        try {
            url = new URL(urlStr);
            connection = (HttpURLConnection) url.openConnection();    //新建链接实例
            connection.setConnectTimeout(20000);    //设置链接超时时间，单位毫秒
            connection.setReadTimeout(20000);        //设置读取数据超时时间，单位毫秒
            connection.setDoOutput(true);        //是否打开输出流true|false
            connection.setDoInput(true);        //是否打开输入流true|false
            connection.setRequestMethod("GET");        // 提交方式get|post
            connection.setUseCaches(false);        // 是否加入缓存true|false
            connection.connect();    //打开链接端口

            BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream(), encoding));    // 往对端写完数据对端服务器返回数据。以BufferedReader流来读取

            while ((line = reader.readLine()) != null) {
                text += line + "\n";
            }
            reader.close();

            JSONObject jsonObject = JSONObject.parseObject(text);
            // 省份
            regionPro = jsonObject.get("pro").toString();
            map.put("regionPro", regionPro);
            // 市区
            regionCity = jsonObject.get("city").toString();
            map.put("regionCity", regionCity);

        } catch (MalformedURLException e) {
            e.printStackTrace();
        } finally {
            if (connection != null) {
                connection.disconnect();    //关闭连接
            }
        }
        return map;
    }

    private String source(String link) {
        if (link.contains("www.baidu.com")) {
            return "1";
        }
        if (link.contains("www.so.com")) {
            return "2";
        }
        if (link.contains("www.google.cn")) {
            return "3";
        }
        if (link.contains("m.sm.cn")) {
            return "4";
        }
        if (link.contains("toutiao.com")) {
            return "5";
        }
        if (link.contains("www.sogou.com")) {
            return "6";
        }
        if (link.contains("servicewechat.com")) {
            return "7";
        }
        if (link.contains("www.caimei365.com")) {
            return "0";
        }
        if (link.contains("zzjtest.gz.aeert.com")) {
            return "0";
        }
        if (link.contains("material") && link.contains("caimei365.com")) { //https://material-b.caimei365.com
            return "8";
        }
        return null;
    }

    private boolean boolReptiles(String userAgent) {
        if (userAgent.contains("serpstatbot") || userAgent.contains("spider") || userAgent.contains("Googlebot") || userAgent.contains("bingbot")
                || userAgent.contains("SeznamBot") || userAgent.contains("YisouSpider") || userAgent.contains("petalbot") || userAgent.contains("YandexBot")
                || userAgent.contains("AhrefsBot") || userAgent.contains("BingPreview/1.0b")) {
            return true;
        }
        return false;
    }
}
