package com.caimei365.tools.service.impl;

import com.caimei365.tools.mapper.CmBehaviorRecordMapper;
import com.caimei365.tools.model.po.CmCouponAssociatePo;
import com.caimei365.tools.model.po.CmCouponPo;
import com.caimei365.tools.model.po.CmVipCouponPo;
import com.caimei365.tools.model.po.CmVipCouponRelation;
import com.caimei365.tools.service.CmVipCouponService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/2/1
 */
@Slf4j
@Service
public class CmVipCouponServiceImpl implements CmVipCouponService {
    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
    private static Calendar calendar = Calendar.getInstance();

    @Autowired
    private CmBehaviorRecordMapper cmBehaviorRecordMapper;

    /**
     * 放开优惠券，创建下季度优惠券
     */
    @Override
    public void openCoupon() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM");
        Date date = new Date();
        String startTime = simpleDateFormat.format(date);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, 3);
        Date time = calendar.getTime();
        String endTime = simpleDateFormat.format(time);
        CmVipCouponPo vipCouponPo = new CmVipCouponPo();

        CmVipCouponPo cmVipCouponPo = cmBehaviorRecordMapper.selSvipCoupon(startTime, endTime);
        // 将本季度优惠券开放
        cmBehaviorRecordMapper.updateSvipCoupon(cmVipCouponPo);
        // 生成下一季度优惠券
        vipCouponPo.setUseTime(endTime);
        vipCouponPo.setStatus("1");
        vipCouponPo.setUpdateTime(date);
        vipCouponPo.setDelFlag("1");
        cmBehaviorRecordMapper.insertVipCouponMonth(vipCouponPo);
        // 修改专属优惠券配置中优惠券的有效时间
        calendar.setTime(time);
        calendar.add(Calendar.MONTH, 2);
        Date times = calendar.getTime();
        String formatTime = simpleDateFormat.format(times);
        Date startDate = getMinDay(endTime);
        Date endDate = getMaxDay(formatTime);
        List<CmCouponPo> cmCouponPoList = cmBehaviorRecordMapper.selCoupon();
        for (CmCouponPo cmCouponPo : cmCouponPoList) {
            int couponId = cmCouponPo.getId();  // 获取专属优惠券id
            cmCouponPo.setConfigure(0);
            // 修改优惠券配置属性
            cmBehaviorRecordMapper.updateCoupon(cmCouponPo);
            // 插入专属优惠券配置中优惠券
            cmCouponPo.setName("超级会员优惠券" + endTime);
            cmCouponPo.setStartDate(startDate);
            cmCouponPo.setEndDate(endDate);
            cmCouponPo.setConfigure(1);
            cmCouponPo.setCreateDate(new Date());
            cmCouponPo.setDelFlag("0");
            cmBehaviorRecordMapper.insertCoupon(cmCouponPo);

            /*
             * vip优惠券月份关系
             */
            CmVipCouponRelation relation = new CmVipCouponRelation();
            relation.setCouponId(cmCouponPo.getId());
            relation.setMontId(vipCouponPo.getId());
            relation.setDelFlag("0");
            relation.setUpdateTime(date);
            cmBehaviorRecordMapper.insertRelation(relation);

            // 插入新优惠券商品
            List<CmCouponAssociatePo> associatePoList = cmBehaviorRecordMapper.selCouponPro(couponId);
            log.info("couponId=======>"+couponId);
            if (associatePoList.size() != 0) {
                for (CmCouponAssociatePo couponAssociatePo : associatePoList) {
                    log.info("associatePoList =======>"+couponAssociatePo);
                    couponAssociatePo.setCouponId(cmCouponPo.getId());
                    cmBehaviorRecordMapper.insertCouponAssociate(couponAssociatePo);
                }
            }
        }
        log.info("========优惠券创建成功========");
    }

    /**
    * 输入日期字符串比如201703，返回当月第一天的Date
    */
    public static Date getMinDay(String month) {
        try {
            Date nowDate = sdf.parse(month);
            calendar = Calendar.getInstance();
            calendar.setTime(nowDate);
            calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
            return calendar.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
    * 输入日期字符串，返回下个月1号0点
    */
    public static Date getMaxDay(String month) {
        try {
            Date nowDate = sdf.parse(month);
            calendar = Calendar.getInstance();
            calendar.setTime(nowDate);
            calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
            calendar.add(Calendar.DAY_OF_MONTH, 1);
            return calendar.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }
}
