package com.caimei365.tools.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.caimei365.tools.model.dto.MessageDto;
import com.caimei365.tools.model.dto.message.*;
import com.caimei365.tools.model.dto.message.enums.MessageType;
import com.caimei365.tools.service.MessageSender;
import com.caimei365.tools.service.RocketMqService;
import com.caimei365.tools.utils.EmailUtil;
import com.caimei365.tools.utils.InsideMessageUtil;
import com.caimei365.tools.utils.SmsUtil;
import com.caimei365.tools.utils.WeChatUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
@Slf4j
public class MessageSenderImpl implements MessageSender {
    @Resource
    private EmailUtil emailSender;
    @Resource
    private WeChatUtil weChatSender;
    @Resource
    private InsideMessageUtil messageUtil;
    @Resource
    private RocketMqService mqService;

    @Override
    public void messageSend(MessageModel model) {
        MessageType code = model.code();
        switch (code) {
            case SYS_MESSAGE:
                sendSMS(model);
                break;
            case EMAILS:
                sendEmail(model);
                break;
            case WEB_INSIDE_MESSAGE:
                sendInside(model);
                break;
            case WECHAT_TEMPLATE_MESSAGE:
                sendWechat(model);
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + code);
        }
    }

    public void sendSMS(MessageModel model) {
        String info = JSONObject.toJSONString(model.info());        SmsMessage smsMessage = JSONObject.parseObject(info, SmsMessage.class);
        if (null != model.mqInfo() && StringUtils.isNotBlank(model.mqInfo().topic())) {
            mqService.sendCommonMessage(setMessageDto(model));
        } else {
            SmsUtil.sendSms(smsMessage.type(), smsMessage.mobile(), smsMessage.smsMessageContent());
        }
    }

    public void sendEmail(MessageModel model) {
        if (null != model.mqInfo() && StringUtils.isNotBlank(model.mqInfo().topic())) {
            mqService.sendCommonMessage(setMessageDto(model));
        } else {
            String info = JSONObject.toJSONString(model.info());
            EmailMessage emailMessage = JSONObject.parseObject(info, EmailMessage.class);
            emailSender.sendSimpleMail(emailMessage);
        }
    }

    public void sendInside(MessageModel model) {
        if (null != model.mqInfo() && StringUtils.isNotBlank(model.mqInfo().topic())) {
            mqService.sendCommonMessage(setMessageDto(model));
        } else {
            String info = JSONObject.toJSONString(model.info());
            InsideMessage insideMessage = JSONObject.parseObject(info, InsideMessage.class);
            messageUtil.sendInsideMessage(insideMessage);
        }
    }

    public void sendWechat(MessageModel model) {
        if (null != model.mqInfo() && StringUtils.isNotBlank(model.mqInfo().topic())) {
            mqService.sendCommonMessage(setMessageDto(model));
        } else {
            String info = JSONObject.toJSONString(model.info());
            WechatTemplateMessage wechatTemplateMessage = JSONObject.parseObject(info, WechatTemplateMessage.class);
            weChatSender.sendMessage(wechatTemplateMessage);
        }
    }

    private MessageDto setMessageDto(MessageModel model) {
        MessageDto messageDto = new MessageDto();
        MqInfo mqInfo = model.mqInfo();
        messageDto = messageDto.async(mqInfo.async()).tag(mqInfo.tag()).topic(mqInfo.topic()).
                delay(mqInfo.delay()).content(JSONObject.toJSONString(model));
        return messageDto;
    }
}
