package com.caimei365.tools.utils;

import com.github.tobato.fastdfs.domain.fdfs.StorePath;
import com.github.tobato.fastdfs.domain.proto.storage.DownloadByteArray;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.*;
import java.util.UUID;

import static com.sun.jmx.snmp.EnumRowStatus.active;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/10/25
 */
@Component
public class FastDfsUtil {
    protected Logger logger = LoggerFactory.getLogger(getClass());
    @Value("${spring.cloud.config.profile}")
    private String profile;
    @Resource
    private FastFileStorageClient storageClient;


    // 文件上传
    public String uploadFile(String path) throws FileNotFoundException {
        File file = new File(path);
        InputStream input = new FileInputStream(file);
        long size = FileUtils.sizeOf(file);
        String name = file.getName();
        String fileName = name.substring(name.lastIndexOf("/") + 1);
        StorePath storePath = storageClient.uploadFile(input, size, FilenameUtils.getExtension(fileName), null);
        return storePath.getFullPath();
    }

    // 文件下载
    public boolean downloadFile(String path, String downloadFilePath) throws IOException {
        File file = new File(downloadFilePath);
        FileOutputStream outputStream = null;
        // fastdfs 文件读取
        String filepath = path.substring(path.lastIndexOf("group1/") + 7);
        DownloadByteArray callback = new DownloadByteArray();
        byte[] content = storageClient.downloadFile("group1", filepath, callback);
        // 数据写入指定文件夹中
        outputStream = new FileOutputStream(file);
        outputStream.write(content);
        return true;
    }
    /**
     * 保存文件到FastDFS
     */
    public String saveFile(ByteArrayInputStream inputStream) throws IOException {
        String randomStr = UUID.randomUUID().toString();
        String name = ".jpeg";
        // 图片暂存本地服务器路径
        String filePath = "/mnt/newdatadrive/data/runtime/cloud-instance/server-tools/tempImage/";
        if ("dev".equals(profile)){
            filePath = "D:\\";
        }
        filePath += randomStr + name;
        FileOutputStream outputStream = new FileOutputStream(filePath);
        int i = 0;
        byte[] buffer = new byte[200];
        while ((i = inputStream.read(buffer)) != -1) {
            outputStream.write(buffer, 0, i);
        }
        outputStream.flush();
        outputStream.close();
        inputStream.close();
        // 临时图片
        File tempFile = new File(filePath);
        logger.info("【图片上传】>>>>>>>>>>>>>>>>图片临时路径:" + filePath);
        String fileUrl = uploadFile(filePath);
        // 删除临时图片
        boolean delete = tempFile.delete();
        logger.info("【图片上传】>>>>>>>>>>>>>>>>删除临时图片:" + delete);
        return fileUrl;
    }

}
